/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <pargal/algo_util.hpp>
#include <pargal/moab_tag_iterator.hpp>
#include <pargal/moab_tag_chunk.hpp>
#include <pargal/moab_tag_counter.hpp>
#include <memory>
#include "moab_tag_minmax.hpp"
#include "moab_tag_reduce.hpp"

namespace pargal 
{
//! computes the average of a variable's given dimension at all other dimensions

template<typename T>
void dim_min_n_single_dim(pcvar& var, pcvar& var_min, const size_t odim)
{
    bool has_missing_val = var.has_missing_val();
    double missing_val = var.get_missing_val();
    bool is_file_var = !var.is_usr_created();

	std::map<util::moab_tag_chunk::key_type, util::moab_tag_chunk, util::moab_tag_chunk::key_compare_functor> red_chunks;

	pcvar::iterator var_citer = var.get_chunk_iterator(odim);
	pcvar::red_iterator var_min_citer = var_min.get_red_chunk_iterator(odim);

	for(var_citer.chbegin(), var_min_citer.chbegin();
		(!var_citer.chend()) && (!var_min_citer.chend());
		++var_citer, ++var_min_citer){

		util::moab_tag_chunk var_tc = *var_citer;
		util::moab_tag_chunk var_min_tc = *var_min_citer;

		red_chunks.insert(std::pair<util::moab_tag_chunk::key_type, util::moab_tag_chunk>(
			var_min_tc.get_key(), var_min_tc));

		for(util::moab_tag_chunk::iterator var_seq_iter = var_tc.begin(),
				var_min_seq_iter = var_min_tc.begin();
				(var_seq_iter != var_tc.end()) &&
				(var_min_seq_iter != var_min_tc.end());
				++var_seq_iter, ++var_min_seq_iter){
			util::moab_tag_chunk_seq& var_seq = *var_seq_iter;
			util::moab_tag_chunk_seq& var_min_seq = *var_min_seq_iter;	

			algorithm::util::distributed_min(var_seq, var_min_seq,
				typename util::dim_traits<T>::dim_iterator_category(),
				typename util::dim_traits<T>::dim_red_iterator_category());
		}
		if(var_tc.needs_reduce()){
			algorithm::util::dim_reduce<T>(red_chunks, var_min, algorithm::util::PARGAL_RED_OP_DIM_MIN, odim);
			red_chunks.clear();
		}
	}
}

void dim_min_n(pcvar& var, pcvar& var_min, const std::vector<std::size_t>& dims, bool keep)
{
	std::size_t sz = dims.size();
	pcvar *in_var = &var;
	std::vector<pcvar *> out_var(2);
	out_var[0] = &var_min;
	out_var[1] = NULL;

	var_min.init(std::numeric_limits<double>::max());

	assert(!dims.empty());
	if(sz > 1){
		// Use a single temp variable for calcs with > 1 dims
		// FIXME: Is the naming of the temp var good enough?
		std::string var_tmp_name("_PARGAL_"+var_min.name());
		out_var[1] = new pcvar(var_tmp_name, var_min.type(),
							var_min.get_dims(), var_min.get_finfo(),
							var_min.get_tsteps(), true,
							var_min.get_loc(), false, var_min.get_missing_val());
		out_var[1]->init(std::numeric_limits<double>::max());
	}

	size_t out_var_idx = ((sz % 2) == 0) ? 1 : 0;
	for(size_t i=0; i<sz; i++){
		PARGAL_VAR_DIM_VOID_FUNC_DISPATCH(in_var->get_dim_type(dims[i]), dim_min_n_single_dim, *in_var, *(out_var[out_var_idx]), dims[i]);
		// Use the result as input for next calc
		in_var = out_var[out_var_idx];
		out_var_idx = (out_var_idx + 1) % 2;
	}
}

} // namespace pargal

