/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_MOAB_TAG_DIVIDE_HPP_INCLUDED
#define PARGAL_MOAB_TAG_DIVIDE_HPP_INCLUDED

/*
#include <pargal/pcvar.hpp>
#include <pargal/moab_tag_iterator.hpp>
#include <pargal/moab_tag_chunk.hpp>
#include <pargal/moab_tag_counter.hpp>
*/
#include <cstddef>

namespace pargal{
namespace algorithm{
namespace util{

template<typename dim_iter, typename red_dim_iter>
inline void distributed_divide_m(pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter)
{
	throw pargal::pargal_except("Error: Unknown iterator, cannot perform distributed sum", __FILE__, __LINE__, "Unknown");
}

template<>
inline void distributed_divide_m<pargal::util::moab_tag_sparse_skip_dim_iterator,
	pargal::util::moab_tag_sparse_static_dim_iterator>(
	pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter)
{
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	std::size_t skip_cnt = var_seq.get_skip_count();
	std::size_t skip_dist = var_seq.get_skip_dist();
	std::size_t reset_cnt = var_seq.get_reset_count();
	std::size_t reset_dist = var_seq.get_reset_dist();
	double mval = var_seq.get_missing_val();

	for(; (viter != var_seq.end()) && (citer != counter.end());){
		pargal::util::moab_tag_chunk_seq::iterator reset_viter = viter;
		for(std::size_t i=0; 
			(i < reset_cnt) &&
			(viter != var_seq.end()) &&
			(citer != counter.end());
			 i++, ++viter){
			if(*citer > 0){
				*viter /= *citer;	
			}
		}
		viter = reset_viter;
		std::advance(viter, skip_cnt);
		++citer;
	}
}

template<>
inline void distributed_divide_m<pargal::util::moab_tag_gen_dim_iterator,
	pargal::util::moab_tag_reset_dim_iterator>(
	pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter)
{
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	double mval = var_seq.get_missing_val();
	for(; (viter != var_seq.end()) && (citer != counter.end()); ++viter, ++citer){
		if(*citer > 0){
			*viter /= *citer;
		}
	}
}

template<>
inline void distributed_divide_m<pargal::util::moab_tag_gen_dim_iterator,
	pargal::util::moab_tag_static_skip_dim_iterator>(
	pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter)
{
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	std::size_t skip_dist = var_seq.get_skip_dist();
	double mval = var_seq.get_missing_val();

	for(; (viter != var_seq.end()) && (citer != counter.end()); ++citer){
		if(*citer > 0){
			*viter /= *citer;
		}
		std::advance(viter, skip_dist);
	}
}

template<>
inline void distributed_divide_m<pargal::util::moab_tag_gen_dim_iterator,
	pargal::util::moab_tag_reset_skip_dim_iterator>(
	pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter)
{
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	std::size_t reset_cnt = var_seq.get_reset_count();
	std::size_t skip_dist = var_seq.get_skip_dist();
	double mval = var_seq.get_missing_val();

	for(;
		(viter != var_seq.end()) &&
		(citer != counter.end())
		;){
		pargal::util::moab_tag_chunk_seq::iterator reset_viter = viter;
		for(size_t i=0;
			(viter != var_seq.end()) &&
			(citer != counter.end()) &&
			(i < reset_cnt);
			i++, ++viter, ++citer){
			if(*citer > 0){
				*viter /= *citer;
			}
		}
		viter = reset_viter;
		std::advance(viter, skip_dist);
	}
}

inline void distributed_divide(pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter,
	pargal::util::moab_tag_sparse_skip_dim_iterator,
	pargal::util::moab_tag_sparse_static_dim_iterator)
{
	if(var_seq.has_missing_val()){
		distributed_divide_m<pargal::util::moab_tag_sparse_skip_dim_iterator,
			pargal::util::moab_tag_sparse_static_dim_iterator>(var_seq, counter);
		return;
	}
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	std::size_t skip_cnt = var_seq.get_skip_count();
	std::size_t skip_dist = var_seq.get_skip_dist();
	std::size_t reset_cnt = var_seq.get_reset_count();
	std::size_t reset_dist = var_seq.get_reset_dist();

	for(; (viter != var_seq.end()) && (citer != counter.end());){
		pargal::util::moab_tag_chunk_seq::iterator reset_viter = viter;
		for(std::size_t i=0; 
			(i < reset_cnt) &&
			(viter != var_seq.end()) &&
			(citer != counter.end());
			 i++, ++viter){
			*viter /= *citer;	
		}
		viter = reset_viter;
		std::advance(viter, skip_cnt);
		++citer;
	}
}

inline void distributed_divide(pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter,
	pargal::util::moab_tag_gen_dim_iterator,
	pargal::util::moab_tag_reset_dim_iterator)
{
	if(var_seq.has_missing_val()){
		distributed_divide_m<pargal::util::moab_tag_gen_dim_iterator,
			pargal::util::moab_tag_reset_dim_iterator>(var_seq, counter);
		return;
	}
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	for(; (viter != var_seq.end()) && (citer != counter.end()); ++viter, ++citer){
		*viter /= *citer;
	}
}

inline void distributed_divide(pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter,
	pargal::util::moab_tag_gen_dim_iterator,
	pargal::util::moab_tag_static_skip_dim_iterator)
{
	if(var_seq.has_missing_val()){
		distributed_divide_m<pargal::util::moab_tag_gen_dim_iterator,
			pargal::util::moab_tag_static_skip_dim_iterator>(
			var_seq, counter);
		return;
	}
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	std::size_t skip_dist = var_seq.get_skip_dist();

	for(; (viter != var_seq.end()) && (citer != counter.end()); ++citer){
		*viter /= *citer;
		std::advance(viter, skip_dist);
	}
}

inline void distributed_divide(pargal::util::moab_tag_chunk_seq &var_seq,
	pargal::util::moab_tag_seq_counter &counter,
	pargal::util::moab_tag_gen_dim_iterator,
	pargal::util::moab_tag_reset_skip_dim_iterator)
{
	if(var_seq.has_missing_val()){
		distributed_divide_m<pargal::util::moab_tag_gen_dim_iterator,
			pargal::util::moab_tag_reset_skip_dim_iterator>(
			var_seq, counter);
		return;
	}
	pargal::util::moab_tag_chunk_seq::iterator viter = var_seq.begin();
	pargal::util::moab_tag_seq_counter::iterator citer = counter.begin();
	std::size_t reset_cnt = var_seq.get_reset_count();
	std::size_t skip_dist = var_seq.get_skip_dist();
	for(;
		(viter != var_seq.end()) &&
		(citer != counter.end())
		;){
		pargal::util::moab_tag_chunk_seq::iterator reset_viter = viter;
		for(size_t i=0;
			(viter != var_seq.end()) &&
			(citer != counter.end()) &&
			(i < reset_cnt);
			i++, ++viter, ++citer){
			*viter /= *citer;
		}
		viter = reset_viter;
		std::advance(viter, skip_dist);
	}
}

} //namespace util
} //namespace algorithm
} // namespace pargal

#endif // PARGAL_MOAB_TAG_DIVIDE_HPP_INCLUDED
