/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <pargal/moab_tag_chunk.hpp>
#include <pargal/pcvar.hpp>
#include <pargal/moab_tag_dim.hpp>

// moab_tag_chunk seq classes

pargal::util::moab_tag_chunk_seq::moab_tag_chunk_seq(const pargal::util::moab_tag_chunk_seq &other): begin_(other.begin_), end_(other.end_), has_missing_val_(other.has_missing_val_), missing_val_(other.missing_val_)
{
	mb_tag_dim_ = std::auto_ptr<moab_tag_dim> (other.mb_tag_dim_->clone());
}

// Moab tag chunk classes

pargal::util::moab_tag_chunk::moab_tag_chunk(std::list<std::pair<double *, double *> >&chunk_seqs, const std::size_t odim, pargal::pcvar &var, bool needs_reduce, bool is_init): sz_(0), odim_(odim), has_missing_val_(false), missing_val_(0.0), needs_reduce_(needs_reduce), is_init_(is_init)
{
	std::auto_ptr<moab_tag_dim> mb_tag_dim = std::auto_ptr<moab_tag_dim> (create_moab_tag_dim(var, odim));
	has_missing_val_ = var.has_missing_val(); 
	if(has_missing_val_){
		missing_val_ = var.get_missing_val();
	}
	for(std::list<std::pair<double *, double *> >::iterator seq_iter = chunk_seqs.begin();
			seq_iter != chunk_seqs.end(); ++seq_iter){
		moab_tag_chunk_seq::iterator begin, end;
		begin = (*seq_iter).first;
		end = (*seq_iter).second;

		sz_ += end - begin;
		chunk_seqs_.push_back(moab_tag_chunk_seq(begin, end, 
				std::auto_ptr<moab_tag_dim>(mb_tag_dim->clone()), has_missing_val_,
				missing_val_));
	}
}
