/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_LOGGER_HPP_INCLUDED
#define PARGAL_LOGGER_HPP_INCLUDED

#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <iterator>

namespace pargal 
{
  class Logger 
  {    
  public:
    //! constructor
    explicit Logger(std::ostream& os);

    //! copy constructor
    Logger(const Logger& rhs);
    
    //! destructor
    ~Logger();
    
    //! log a single message
    void log(const std::string& msg);
 
    //! log the file name, line number and function name
    void log(const std::string& sfile, const size_t sline, const std::string& sfunc);
    
    //! flush the logger to std::cout
    void flush();        
    
  private:
    Logger& operator=(const Logger& rhs);
    
  private:
    //! data members
    std::ostream& log_ostream;
    std::vector<std::string> log_msgs;    
  };
  
  inline void 
  Logger::log(const std::string& msg) 
  {
#ifndef NDEBUG
    log_msgs.push_back(msg);
#endif
  }
  
  inline void 
  Logger::log(const std::string& sfile, const size_t sline, const std::string& sfunc)
  {
#ifndef NDEBUG
    std::stringstream s;
    s << sfile << " : line " << sline << "\n"
      << "     " << sfunc << "\n";
    log_msgs.push_back(s.str());    
#endif
  }
  
  inline void 
  Logger::flush() 
  {
#ifndef NDEBUG
    std::copy(log_msgs.rbegin(), log_msgs.rend(), std::ostream_iterator<std::string>(log_ostream, ""));
    std::vector<std::string>().swap(log_msgs);
#endif
  }
} // namespace pargal

#endif
