/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_ALGORITHM_UTIL_IMPL_HPP_INCLUDED
#define PARGAL_ALGORITHM_UTIL_IMPL_HPP_INCLUDED

#include <pargal/algorithm.hpp>
#include <pargal/exception.hpp>

namespace pargal 
{

	//FIXME: The struct below are obsolete we need to get rid of them
  struct max_compare {
    bool m_has_missing_val;
    const double m_missing_val;
    max_compare(bool has_missing=false, const double val=0) : 
      m_has_missing_val(has_missing), m_missing_val(val) {}
    
    bool operator()(double lhs, double rhs);   
  };
  
  struct min_compare {
    bool m_has_missing_val;
    const double m_missing_val;
    min_compare(bool has_missing=false, const double val=0) : 
      m_has_missing_val(has_missing), m_missing_val(val) {}
    bool operator()(double lhs, double rhs);  
  };
  
  struct fn_max {
    const max_compare& comp;
    fn_max(const max_compare& _comp) : comp(_comp) {}
    const double& operator()(const double& lhs, const double& rhs) {
      return std::max(lhs, rhs, comp);
    }
  };

  struct fn_min {
    const min_compare& comp;
    fn_min(const min_compare& _comp) : comp(_comp) {}
    const double& operator()(const double& lhs, const double& rhs) {
      return std::min(lhs, rhs, comp);
    }
  };
  
  struct fn_plus {
    bool m_has_missing_val;
    const double m_missing_val;
    std::size_t counter;
    fn_plus(bool has_missing=false, const double val=0) : 
      m_has_missing_val(has_missing), m_missing_val(val), counter(0) {}    
    double operator()(const double& lhs, const double& rhs);
  };

namespace algorithm{
namespace util{

struct missing_val_comp{
	missing_val_comp(double missing_val):mval_(missing_val){}
	bool operator()(double arg1, double arg2);
	double mval_;
};

inline bool missing_val_comp::operator() (double arg1, double arg2)
{
	if((arg1 != mval_) && (arg2 != mval_)){
		return (arg1 < arg2);
	}
	else if(arg2 == mval_){
		return true;
	}
	else{
		// (arg1 == mval_) AND (arg2 != mval_)
		return false;
	}
}

} // namespace algorithm
} // namespace util

  //! computes the maximum of a variable's given dimension at all other dimensions
  void dim_max_min_n_single_dim_core(pcvar& var, pcvar& var_max, 
				     const std::size_t dim, MPI_Op mpi_op, bool keep);
  
  //! computes the maximum of a variable's given dimension at all other dimensions
  void dim_max_min_n_core(pcvar& var, pcvar& var_avg, const std::vector<std::size_t>& dims,
			  MPI_Op mpi_op, bool keep);
}

#endif
