/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_MOAB_TAG_ITERATOR_HPP_INCLUDED
#define PARGAL_MOAB_TAG_ITERATOR_HPP_INCLUDED

#include <vector>
#include <cstddef>

namespace pargal{

// Fwd decl for pcvar
class pcvar;
namespace util{

// Fwd decl for moab_tag_chunk
class moab_tag_chunk;

// Any iterable class, containing chunk of data, must extend iterable
// The iterators below use this information to iterate the data
class iterable{
	public:
		// Count to remain static
		virtual std::size_t get_static_count(void ) const = 0;
		// Count after which to reset
		virtual std::size_t get_reset_count(void ) const = 0;
		// After reset advance reset_dist
		virtual std::size_t get_reset_dist(void ) const = 0;
		// Count after which to skip
		virtual std::size_t get_skip_count(void ) const = 0;
		// Dist to skip
		virtual std::size_t get_skip_dist(void ) const = 0;
};

// Marker interface class for static iterators
class moab_tag_static_iterator{
};

// Marker interface class for reset iterators
class moab_tag_reset_iterator{
};

// Marker interface class for skip iterators
class moab_tag_skip_iterator{
};

class moab_tag_chunk_iterator{
	public:
		moab_tag_chunk_iterator(pargal::pcvar &pcvar, const std::size_t tstep, const std::size_t odim);
		virtual ~moab_tag_chunk_iterator();
		void chbegin(void );
		bool chend(void );
		moab_tag_chunk operator*();
		virtual void operator++();

	protected:
		pargal::pcvar &pcvar_;
		std::size_t tstep_;
		std::size_t nsteps_, step_sz_, step_offset_;
		// FIXME: Move these counters to red chunk iterator
		int nsteps_before_reduce_, nsteps_before_reduce_counter_;
		std::size_t odim_;
		bool is_tstep_cached_;
		bool is_sparse_var_;
		// We assume a chunk is inited once it is accessed
		// i.e., once we iterate to the next chunk
		std::vector<bool> is_chunk_init_;
		
	private:
};

class moab_tag_reset_chunk_iterator : public moab_tag_chunk_iterator,
		public moab_tag_reset_iterator{
	public:
		moab_tag_reset_chunk_iterator(pargal::pcvar &pcvar, const std::size_t tstep, const std::size_t odim);
		~moab_tag_reset_chunk_iterator();
		void operator++();
};

class moab_tag_dim_iterator{
};

// Dim iterators below...
class moab_tag_sparse_gen_dim_iterator : public moab_tag_dim_iterator{
};

class moab_tag_sparse_skip_dim_iterator : public moab_tag_dim_iterator,
		public moab_tag_skip_iterator{
};

class moab_tag_sparse_static_dim_iterator : public moab_tag_dim_iterator,
		public moab_tag_static_iterator{
};

class moab_tag_gen_dim_iterator : public moab_tag_dim_iterator{
};

class moab_tag_static_skip_dim_iterator : public moab_tag_dim_iterator,
		public moab_tag_static_iterator,
		public moab_tag_skip_iterator{
};

class moab_tag_reset_dim_iterator : public moab_tag_dim_iterator,
		public moab_tag_reset_iterator{
};

class moab_tag_reset_skip_dim_iterator : public moab_tag_dim_iterator,
		public moab_tag_reset_iterator,
		public moab_tag_skip_iterator{
};

} // namespace util
} // namespace pargal

#endif // PARGAL_MOAB_TAG_ITERATOR_HPP_INCLUDED
