/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_ATT_HPP_INCLUDED
#define PARGAL_ATT_HPP_INCLUDED

#include <string>
#include <vector>
#include <map>
#include <set>

namespace pargal 
{  
  class pcatt 
  {
  public:
    //! constructor. val is read from MOAB, which is a character
    //! string, with '\0' terminating each attribute name, ';'
    //! separating the data type and value, and ';' separating one
    //! name/data_type/value from the next. att_len indicates the end
    //! position of each name/data_type/value
    explicit pcatt(const std::string& var_name, 
		   const std::string& val,
		   const std::vector<int>& att_len);
    
    //! return the name of variable the attribute associates
    const std::string& get_var_name() const;
    
    //! return the number of attributes for the variable
    std::size_t size() const;

    //! return the attribute names for the variable
    std::set<std::string> get_att_names() const;

    //! return the type of a given attribute name, which is one of the
    //! "char", "int", "double", "float", "short"
    const std::string& get_data_type(const std::string& att_name) const;

    //! get the vals of the attribute for a given attribute name, the
    //! function should be called according to attribute's data type
    void get_vals(const std::string& att_name, std::string& vals) const;    
    void get_vals(const std::string& att_name, std::vector<double>& vals) const;
    void get_vals(const std::string& att_name, std::vector<float>& vals) const;
    void get_vals(const std::string& att_name, std::vector<int>& vals) const;
    void get_vals(const std::string& att_name, std::vector<short>& vals) const;

  private:    
    class att_data 
    {
    public:
      att_data(const std::string& data_type = std::string(), 
	       const std::string& val = std::string()) :
	att_data_type(data_type), att_val(val)
      {}

      const std::string& get_data_type() const
      {
	return att_data_type;
      }

      const std::string& get_val() const
      {
	return att_val;
      }

    private:
      std::string att_data_type;
      std::string att_val;
    };

    const std::map<std::string, att_data>& get_data() const
    { 
      return m_atts;
    }

  private:
    std::string m_var_name;
    std::string m_val;
    //mapping from attribute name to datatype and value
    std::map<std::string, att_data> m_atts;
  };
} // namespace pargal

#endif
