/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_DIM_HPP_INCLUDED
#define PARGAL_DIM_HPP_INCLUDED

#include <string>

namespace pargal 
{
  class pcdim 
  {
  public:
    //! return the name of the dimension if it exists, 0 otherwise.
    inline std::string name() const;

    //! return the dimension size
    inline std::size_t size() const;
    
    //! return true for unlimited dimension
    inline bool is_unlimited() const;
    
    //! constructor
    inline explicit pcdim(const std::string& name = std::string(), 
			  const std::size_t sz = 0, 
			  bool is_unlimited = false);
    
    //! copy constructor
    inline pcdim(const pcdim& rhs);

    // set the dimension size
    inline void set_size(const std::size_t& sz);
    
  private:
    std::string m_name;
    std::size_t m_size;
    bool m_is_unlimited;
    
    friend class fileinfo;
  };

  pcdim::pcdim(const std::string& name, 
	       const std::size_t sz, 
	       bool is_unlimited) : 
    m_name(name),
    m_size(sz),
    m_is_unlimited(is_unlimited)
  {}
        
  //copy constructor
  pcdim::pcdim(const pcdim& rhs)
  {
    m_name = rhs.m_name;
    m_size = rhs.m_size;
    m_is_unlimited = rhs.m_is_unlimited;
  }

  //! return the name of the dimension if it exists, 0 otherwise.
  std::string 
  pcdim::name() const
  {
    return m_name;
  }

  //! return the dimension size
  std::size_t
  pcdim::size() const
  {
    return m_size;
  }
    
  //! return true for unlimited dimension
  bool 
  pcdim::is_unlimited() const
  {
    return m_is_unlimited;
  }  

  //! set the dimension size
  void
  pcdim::set_size(const std::size_t& sz)
  {
    m_size = sz;
  }
} // namespace pargal

#endif
