/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_TIMER_HPP_INCLUDED
#define PARGAL_TIMER_HPP_INCLUDED

#include "mpi.h"

namespace pargal 
{  
  class Timer 
  {
  public:
    //! constructor
    explicit Timer();
    
    //! destructor
    ~Timer();
    
    //! reset timer
    void restart();
    
    //! returns an elapsed wall clock time in seconds (double
    //! precision) on the root.
    double elapsed(int root = 0) const;
    
  private:
    Timer& operator=(const Timer& rhs);
    Timer(const Timer& rhs);
    
  private:
    //! data members
    double m_start;
  };
  
  inline void 
  Timer::restart()
  {
    m_start = MPI_Wtime();
  }
  
  inline double
  Timer::elapsed(int root) const
  {
    double m_end = MPI_Wtime();
    double elapsed = m_end - m_start;
    double result = 0;
    MPI_Reduce(&elapsed, &result, 1, MPI_DOUBLE, MPI_MAX, root, MPI_COMM_WORLD);
    return result;
  }  
} // namespace pargal
#endif
