/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "pargal_tutil.hpp"

using namespace pargal;
using namespace moab;

std::string var_avg_name("Tavg");

PARGAL_TUTIL_AUTO_MPI_TEST_CASE (avg_lat_singlef_test)
{
  int& argc = boost::unit_test::framework::master_test_suite().argc;
  char**& argv = boost::unit_test::framework::master_test_suite().argv;
  
  std::string file_name(argv[1]);
  std::string ncl_avg_output(argv[2]);
  std::string var_name(argv[3]);

  // create moab instance   
  Core moab;
  Interface& mb = moab;
  moab::ParallelComm mb_pcomm(&mb, MPI_COMM_WORLD);

  // create meshset
  EntityHandle file_set;
  ErrorCode rval = mb.create_meshset(MESHSET_SET, file_set);
  if (rval != MB_SUCCESS) 
throw pargal_except("Error: " + mb.get_error_string(rval),
			__FILE__, __LINE__, __PRETTY_FUNCTION__);

  // load file metadata
  std::vector<std::string> file_names;
  file_names.push_back(file_name);  
  std::string part_method("SQIJ");
  fileinfo finfo(file_names, mb, mb_pcomm, file_set, part_method);

  std::stringstream opts;  
  opts << "PARALLEL=READ_PART;PARTITION;PARALLEL_RESOLVE_SHARED_ENTS;PARALLEL_GHOSTS=2.0.1;PARTITION_METHOD=";
  opts << part_method;
  opts << ";VARIABLE=";
  rval = mb.load_file(file_name.c_str(), &file_set, opts.str().c_str());
  if (rval != moab::MB_SUCCESS) 
throw pargal_except("Error: " + mb.get_error_string(rval),
			__FILE__, __LINE__, __PRETTY_FUNCTION__);

  std::size_t num_tsteps = finfo.get_dim("time").size();
  std::vector<int> tsteps(num_tsteps);
  for (std::size_t i = 0; i != num_tsteps; ++i)
tsteps[i] = i;

  pcvar var(var_name, moab::MB_TYPE_DOUBLE, finfo.get_var_dims(var_name),
	finfo, tsteps, false);
  
  const std::vector<pcdim>& var_avg_dims = finfo.get_var_dims(var_name);
  pcvar var_avg(var_avg_name, moab::MB_TYPE_DOUBLE, 
		var_avg_dims, finfo, tsteps, true,
		var.get_loc(), false, var.get_missing_val());

  //comupte average over lat
  std::vector<std::size_t> dims(1,2);
  dim_avg_n(var, var_avg, dims);

  //gather average
  std::vector<double> avg_result;
  gather(var_avg, tsteps, avg_result);

  std::vector<int> dims_to_check(3);
  /* Check time, lev, lon - lat dim contains invalid values */
  dims_to_check[0] = 0; dims_to_check[1] = 1; dims_to_check[2] = 3;
  
  pargal_tutil::var_dims_check_root(var_avg, dims_to_check, avg_result, ncl_avg_output);
}
