/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "pargal_tutil.hpp"

using namespace pargal;
using namespace moab;

std::string var_max_name("Tmax");

PARGAL_TUTIL_AUTO_MPI_TEST_CASE (max_time_singlef_unstructured_mesh_test)
{
	int& argc = boost::unit_test::framework::master_test_suite().argc;
	char**& argv = boost::unit_test::framework::master_test_suite().argv;

	std::string file_name(argv[1]);
	std::string ncl_max_output(argv[2]);
	std::string var_name(argv[3]);

	// create moab instance   
	Core moab;
	Interface& mb = moab;
	moab::ParallelComm mb_pcomm(&mb, MPI_COMM_WORLD);

	// create meshset
	EntityHandle file_set;
	ErrorCode rval = mb.create_meshset(MESHSET_SET, file_set);
	if (rval != MB_SUCCESS){ 
		throw pargal_except("Error: " + mb.get_error_string(rval),
				__FILE__, __LINE__, __PRETTY_FUNCTION__);
	}

	// load file metadata
	std::vector<std::string> file_names;
	file_names.push_back(file_name);  
	std::string part_method("TRIVIAL_PARTITION");
	fileinfo finfo(file_names, mb, mb_pcomm, file_set, part_method);

	std::stringstream opts;  
	opts << "PARALLEL=READ_PART;PARTITION;PARALLEL_RESOLVE_SHARED_ENTS;GATHER_SET=0;PARTITION_METHOD=";
	opts << part_method;
	opts << ";VARIABLE=";
	rval = mb.load_file(file_name.c_str(), &file_set, opts.str().c_str());
	if (rval != moab::MB_SUCCESS){ 
		throw pargal_except("Error: " + mb.get_error_string(rval),
				__FILE__, __LINE__, __PRETTY_FUNCTION__);
	}

	const std::vector<pcdim>& var_dims = finfo.get_var_dims(var_name);
	int time_dim_idx = -1, dim_idx = 0;
	std::size_t ndims = var_dims.size();
	std::size_t num_tsteps = 1;
	for(std::vector<pcdim>::const_iterator iter = var_dims.begin();
			iter != var_dims.end(); ++iter, dim_idx++){
		if(iter->name() == "time"){
			time_dim_idx = dim_idx;
			num_tsteps = finfo.get_dim("time").size();
			break;
		}
		else if(iter->name() == "Time"){
			time_dim_idx = dim_idx;
			num_tsteps = finfo.get_dim("Time").size();
			break;
		}
	}

	BOOST_REQUIRE(time_dim_idx == 0);

	std::vector<int> tsteps(num_tsteps);
	for (std::size_t i = 0; i != num_tsteps; ++i){
		tsteps[i] = i;
	}

	pcvar var(var_name, moab::MB_TYPE_DOUBLE, var_dims,
				finfo, tsteps, false);

	std::vector<pcdim> var_max_dims(var_dims);
	//FIXME: Assumes time is the left-most dimension
	var_max_dims[time_dim_idx].set_size(1);
	std::vector<int> max_tsteps(1, 0);
	pcvar var_max(var_max_name, moab::MB_TYPE_DOUBLE, 
					var_max_dims, finfo, max_tsteps, true,
					var.get_loc(), false, var.get_missing_val());

	//comupte average over time
	std::vector<std::size_t> dims(1,0);
	dim_max_n(var, var_max, dims);

	//gather average
	std::vector<double> max_result;
	gather(var_max, max_tsteps, max_result);

	std::size_t ndims_to_check = ndims - 1;
	std::vector<int> dims_to_check(ndims_to_check);
	for(std::size_t i=0, j=0;
			(i<ndims) && (j < ndims_to_check); i++){
		if(i != time_dim_idx){
			dims_to_check[j++] = i;
		}
	}

	pargal_tutil::var_dims_check_root(var_max, dims_to_check, max_result, ncl_max_output);
}
