/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "pargal_tutil.hpp"

using namespace pargal;
using namespace moab;

#define STRINGIFY_(X) #X
#define STRINGIFY(X) STRINGIFY_(X)

/*
#ifdef MESHDIR
std::string file_name0(STRINGIFY(MESHDIR) "/CAM/EulSpec/datasets/eul26x48x96.t0.nc");
std::string file_name1(STRINGIFY(MESHDIR) "/CAM/EulSpec/datasets/eul26x48x96.t1.nc");
std::string file_name2(STRINGIFY(MESHDIR) "/CAM/EulSpec/datasets/eul26x48x96.t2.nc");
std::string ncl_sum_output(STRINGIFY(MESHDIR) "/CAM/EulSpec/native/dim_sum_n/eul26x48x96.T.TIME.sum.dat");
#else
std::string file_name0("../../../GridFiles/CAM/EulSpec/datasets/eul26x48x96.t0.nc");
std::string file_name1("../../../GridFiles/CAM/EulSpec/datasets/eul26x48x96.t1.nc");
std::string file_name2("../../../GridFiles/CAM/EulSpec/datasets/eul26x48x96.t2.nc");
std::string ncl_sum_output("../../../GridFiles/CAM/EulSpec/native/dim_sum_n/eul26x48x96.T.TIME.sum.dat");
#endif
*/
std::string var_name("T"); 
std::string var_sum_name("Tsum");

PARGAL_TUTIL_AUTO_MPI_TEST_CASE (sum_time_multif_test)
{
	int& argc = boost::unit_test::framework::master_test_suite().argc;
	char**& argv = boost::unit_test::framework::master_test_suite().argv;


	std::string file_name0(argv[1]);
	std::string file_name1(argv[2]);
	std::string file_name2(argv[3]);
	std::string ncl_sum_output(argv[4]);

	Core moab;
	Interface& mb = moab;
	moab::ParallelComm mb_pcomm(&mb, MPI_COMM_WORLD);

	// create meshset
	EntityHandle file_set;
	ErrorCode rval = mb.create_meshset(MESHSET_SET, file_set);
	if(rval != MB_SUCCESS){ 
		throw pargal_except("Error: " + mb.get_error_string(rval),
			__FILE__, __LINE__, __PRETTY_FUNCTION__);
	}

	// load file metadata  
	std::vector<std::string> file_names;
	file_names.push_back(file_name0);
	file_names.push_back(file_name1);
	file_names.push_back(file_name2);
	std::string part_method("SQIJ");
	fileinfo finfo(file_names, mb, mb_pcomm, file_set, part_method);

	std::stringstream opts;  
	opts << "PARALLEL=READ_PART;PARTITION;PARALLEL_RESOLVE_SHARED_ENTS;PARALLEL_GHOSTS=2.0.1;PARTITION_METHOD=";
	opts << part_method;
	opts << ";VARIABLE=";
	rval = mb.load_file(file_name0.c_str(),
		  &file_set,
		  opts.str().c_str());
	if(rval != moab::MB_SUCCESS){ 
		throw pargal_except("Error: " + mb.get_error_string(rval),
			__FILE__, __LINE__, __PRETTY_FUNCTION__);
	}

	std::size_t num_tsteps = finfo.get_dim("time").size();
	std::vector<int> tsteps(num_tsteps);
	for(std::size_t i = 0; i != num_tsteps; ++i){
		tsteps[i] = i;
	}

    const std::vector<pcdim>& var_pcdims = finfo.get_var_dims(var_name);
    int time_dim_idx = -1, dim_idx = 0;
    std::size_t ndims = var_pcdims.size();
    for(std::vector<pcdim>::const_iterator iter = var_pcdims.begin();
            iter != var_pcdims.end(); ++iter, dim_idx++){
        if(iter->name() == "time"){
            time_dim_idx = dim_idx;
            break;
        }
    }

    BOOST_REQUIRE(time_dim_idx != -1);

	pcvar var(var_name, moab::MB_TYPE_DOUBLE, var_pcdims, finfo, tsteps, false);

	std::vector<pcdim> var_sum_dims(var_pcdims);
	std::vector<std::size_t> var_dims;
	var.get_dims(var_dims);
	std::size_t var_sum_time_dim_sz = 1;
	var_sum_time_dim_sz = std::accumulate(var_dims.begin(),
								var_dims.begin() + time_dim_idx,
								var_sum_time_dim_sz,
								std::multiplies<std::size_t>());

	var_sum_dims[time_dim_idx].set_size(var_sum_time_dim_sz);

	std::vector<int> sum_tsteps(1, 0);
	pcvar var_sum(var_sum_name, moab::MB_TYPE_DOUBLE, 
			var_sum_dims, finfo, sum_tsteps, true,
			var.get_loc(), false, var.get_missing_val());

	//comupte average
	std::vector<std::size_t> dims(1, time_dim_idx);
	dim_sum_n(var, var_sum, dims);

	//gather average
	std::vector<double> sum_result;
	gather(var_sum, sum_tsteps, sum_result);

    // Check dims (lev,lat,lon) etc
    std::size_t ndims_to_check = ndims - 1;
    std::vector<int> dims_to_check(ndims_to_check);
    for(std::size_t i=0, j=0;
            (i<ndims) && (j < ndims_to_check); i++){
        if(i != time_dim_idx){
            dims_to_check[j++] = i;
        }
    }

    pargal_tutil::var_dims_check_root(var_sum, dims_to_check,
		sum_result, ncl_sum_output);
}
