/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PARGAL_TUTIL_HPP_INCLUDED_
#define PARGAL_TUTIL_HPP_INCLUDED_

#include <boost/test/included/unit_test.hpp>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <moab/Core.hpp>
#include <pargal.hpp>
#include "mpi.h"

#define PARGAL_TUTIL_AUTO_MPI_TEST_CASE(tmod_name_)							\
	static void tmod_name_(void );											\
	static void PARGAL_TUTIL_MPI_TEST_CASE ## tmod_name_(void )				\
	{																		\
		try{																\
			tmod_name_();													\
		}																	\
		catch(pargal::pargal_except& e){									\
			e.get_info();													\
			BOOST_ERROR("ParGAL exception: test failed");					\
		}																	\
		catch (std::exception& e){											\
			std::cerr << e.what() << std::endl;								\
			BOOST_ERROR("std exception: test failed");						\
		}																	\
		catch (...){														\
			BOOST_ERROR("Unknown exception: test failed");					\
		}																	\
		MPI_Finalize();														\
	}																		\
	boost::unit_test::test_suite *											\
	init_unit_test_suite(int argc, char *argv[])							\
	{																		\
		int rank;															\
		MPI_Init(&argc, &argv);												\
		MPI_Comm_rank(MPI_COMM_WORLD, &rank);								\
		if(rank != 0){														\
			boost::unit_test::results_reporter::set_level(					\
				boost::unit_test::NO_REPORT);								\
			boost::unit_test::unit_test_log.set_threshold_level(			\
				boost::unit_test::log_nothing);								\
		}																	\
		boost::unit_test::framework::master_test_suite().add(				\
			BOOST_TEST_CASE(&PARGAL_TUTIL_MPI_TEST_CASE ## tmod_name_));	\
		return 0;															\
	}																		\
	static void tmod_name_()
	
#define STRINGIFY_(X) #X
#define STRINGIFY(X) STRINGIFY_(X)

namespace pargal_tutil{
	void var_dims_check_root(pargal::pcvar &pcvar, const std::vector<int> &dims, const std::vector<double> &value, const std::string &cvalue_ifname);
	void var_dims_check_root(pargal::pcvar &pcvar, const std::vector<int> &dims, const std::vector<double> &value, const std::string &cvalue_ifname, double tolerance);
}

#endif /* PARGAL_TUTIL_HPP_INCLUDED_ */
